import os
import json
import typer
import docker_vuln_runner.helper as helper
from docker_vuln_runner.vuln import Vulnenv, vuln_home, get_vuln_objects, all_ports, get_duplicates, vuln_init, check_init, vuln_update, is_initialized, vuln_names, run_vuln, find_vuln_projects, down_vuln, check_docker


typer_app = typer.Typer(add_completion=False, context_settings={
                        "help_option_names": ["-h", "--help"]})


@typer_app.command()
def init():
    """"
    Initialize vuln-runner    
    """
    helper.banner()
    check_docker()
    if is_initialized():
        helper.bold("vulhub already initialized!")
    else:
        helper.log("Init vuln-runner")
        vuln_init()
        helper.success("Vuln initialized!")


@typer_app.command()
def list(silent: bool = typer.Option(False, "--silent", "-s")):
    """
    List the vulhub cves
    """
    helper.banner(silent)
    check_init()
    helper.log("Find vuln repo in {}".format(vuln_home()), silent)
    vulhubs = find_vuln_projects(vuln_home())
    names = vuln_names(vulhubs)
    print("\n".join(names))


@typer_app.command()
def update():
    """
    Update the vulhub git repository
    """
    helper.banner()
    check_init()
    vuln_update()
    helper.success("Vuln updated!")


@typer_app.command()
def run(vulhub_names: str, silent: bool = typer.Option(False, "--silent")):
    """
    Run a list of vuln projects

    Args:
        vulhub_names (str): A comma-separated list of vuln projects
        silent (bool, optional): Silent mode, disable the log to parse the output. Defaults to False
    """
    helper.banner(silent)
    check_init()
    helper.log("Find vuln repos in {}".format(vuln_home()), silent)
    vulhubs = find_vuln_projects(vuln_home())
    vulhub_names = vulhub_names.split(",")
    vulhub_objects = get_vuln_objects(vulhubs, vulhub_names)
    ports = all_ports(vulhub_objects)
    duplicates = get_duplicates(ports)
    if len(duplicates) > 0:
        helper.err(
            "Cannot run, the following ports are duplicates: {}".format(duplicates))

    # Ok, now run
    for vn in vulhub_names:
        helper.log("Run {}".format(vn))
        run_vuln(vulhubs, vn)


@typer_app.command()
def down(vulhub_names: str, silent: bool = typer.Option(False, "--silent")):
    """
    Down a list of vuln projects

    Args:
        vulhub_names (str): A comma-separated list of vuln projects
        silent (bool, optional): Silent mode, disable the log to parse the output. Defaults to False
    """

    helper.banner(silent)
    check_init()
    helper.log("Find vuln repo in {}".format(vuln_home()), silent)
    vulhubs = find_vuln_projects(vuln_home())
    vulhub_names = vulhub_names.split(",")
    vulhub_objects = get_vuln_objects(vulhubs, vulhub_names)
    for vn in vulhub_objects:
        vn.down()


@typer_app.command()
def generate_vulnenv(no_vulns: int, no_env: int = 1):
    """
    Generate <no_env> vulnerable environments composed of <no_vulns> stacks
        Each environment has no port conflicts. It is useful to run different vulhub projects on different machines.
    Args:
        no_vulns (int): The number of vulnerable compose
        no_env (int, optional): The number of vulnerable environments. Defaults to 1.
    """
    helper.banner(True)
    check_init()
    vulhubs = find_vuln_projects(vuln_home())
    vuln_env = Vulnenv(vulhubs)

    try:
        for i in range(0, no_env):
            vuln_env.create_env(i, no_vulns)

    except Exception as e:
        helper.err(e)
    print(vuln_env.to_json())


@typer_app.command()
def run_env(json_file: str, id_env: int):
    """Run an environment taken from a JSON configuration file

    Args:
        json_file (str): The path of the json file
        id_env (int): The environment ID
    """
    helper.banner(True)
    check_init()
    vulhubs = find_vuln_projects(vuln_home())
    if os.path.isfile(json_file):
        with open(json_file) as myfile:
            file_content = myfile.read()
            try:
                json_content = json.loads(file_content)
                envs = json_content['envs']
                if str(id_env) not in envs.keys():
                    helper.err("No valid id_env: {}".format(id_env))
                vuln_list = envs[str(id_env)]
                for v in vuln_list:
                    run_vuln(vulhubs, v['name'])

            except Exception as e:
                # Cannot read the file
                helper.err("Invalid environment json file")
    else:
        helper.err("Please give me a valid file")


@typer_app.command()
def down_env(json_file: str, id_env: int):
    """Down an environment

    Args:
        json_file (str): The path of the json file
        id_env (int): The environment ID
    """

    helper.banner(True)
    check_init()
    vulhubs = find_vuln_projects(vuln_home())
    if os.path.isfile(json_file):
        with open(json_file) as myfile:
            file_content = myfile.read()
            try:
                json_content = json.loads(file_content)
                envs = json_content['envs']
                if str(id_env) not in envs.keys():
                    helper.err("No valid id_env: {}".format(id_env))
                vuln_list = envs[str(id_env)]
                for v in vuln_list:
                    down_vuln(vulhubs, v['name'])

            except Exception as e:
                # Cannot read the file
                helper.err("Invalid environment json file")
    else:
        helper.err("Please give me a valid file")


def app():
    typer_app()



import os
import json
import glob
import git
from pathlib import Path
from yaml import safe_load
from docker_vuln_runner.helper import log, err, get_compose_name
from python_on_whales import DockerClient, docker as docker_cli


class VulnRepo:
    """A vulnerable repository instance
    """

    def __init__(self, name, repo):
        self.name = name
        self.repo = repo

    def home(self):
        return os.path.join(vuln_home(), self.name)


VULN_REPOS = [
    VulnRepo("vulhub", "https://github.com/vulhub/vulhub")
]


def get_duplicates(ports):
    newlist = []  # empty list to hold unique elements from the list
    duplist = []  # empty list to hold the duplicate elements from the list
    for i in ports:
        if i not in newlist:
            newlist.append(i)
        else:
            # this method catches the first duplicate entries, and appends them to the list
            duplist.append(i)
    return duplist


def all_ports(vulns):
    """Returns a list of ports from a list of objects

    Args:
        vulns (list): A list of objects

    Returns:
        list: the list of ports
    """
    ports = []
    for v in vulns:
        ports.extend(v.ports())
    return ports


def vuln_contains(vulns, name):
    """Return true when the vuln is present in the list of vulns

    Args:
        vulns (list): The list of vulns
        name (str): The vuln name

    Returns:
        bool: True when
    """
    for v in vulns:
        if v.equal(name):
            return True
    return False


def find_by_name(vulns, name):
    """Find the

    Args:
        vulns (list<Vuln>): The list of vuln objects
        name (str): The vuln name

    Raises:
        Exception: when the vuln is not found

    Returns:
        Vuln: The found vuln object
    """
    found = [v for v in vulns if v.name == name]
    if not found:
        raise Exception("{} not found".format(name))

    return found[0]


def paths(vulns):
    return [v.path for v in vulns]


def vuln_names(vulns):
    return [v.name for v in vulns]


def find_repo_projects(base_name, base_path, virtual=False):
    ret = []
    for filename in glob.iglob(os.path.join(base_path, '**/docker-compose.yml'), recursive=True):
        if virtual:
            filename = filename.replace(vuln_home(), virtual_vuln_home())
        ret.append(
            Vuln("{}.{}".format(base_name, get_compose_name(filename)), filename))
    return ret


def find_vuln_projects(base_path, virtual=False):
    ret = []
    for V in VULN_REPOS:
        ret = ret + find_repo_projects(V.name, base_path, virtual)

    return ret


def run_vuln(vulns, vuln_name):
    if not vuln_contains(vulns, vuln_name):
        err("{} vuln not found".format(vuln_name))

    vuln_obj = find_by_name(vulns, vuln_name)
    vuln_obj.run()


def down_vuln(vulns, vuln_name):
    if not vuln_contains(vulns, vuln_name):
        err("{} vuln not found".format(vuln_name))

    vuln_obj = find_by_name(vulns, vuln_name)
    vuln_obj.down()


def get_ports(vulns, vuln_name):
    if not vuln_contains(vulns, vuln_name):
        err("{} vuln not found".format(vuln_name))
    vuln_obj = find_by_name(vulns, vuln_name)
    return vuln_obj.ports()


def get_vuln_objects(vulns, names):
    return [v for v in vulns if v.name in names]


def vuln_home():
    """
        Returns the base vuln home
    """
    home_folder = Path.home()
    config_folder = os.path.join(home_folder, '.vulnenv')
    return config_folder


def virtual_vuln_home():
    return "{}".format("VULN_HOME")


def vuln_init():
    if not os.path.isdir(vuln_home()):
        os.mkdir(vuln_home())
    for v in VULN_REPOS:
        log("Clone {} in {}".format(v.repo, v.home()))
        git.Repo.clone_from(v.repo, v.home())


def check_docker():
    # Check if the current user has the permissions to run docker
    # Just run a 'ps' command and check if it throws
    try:
        docker_cli.ps()
    except:
        err("""
        An error occurred while running docker
        Check if the current user is in the 'docker' group
            
        To add it run the following:
        ```
        sudo usermod -aG docker $USER    
        ```    

        """)
    if not docker_cli.compose.is_installed():
        err("""
        Please install 'docker compose' (version 2)

        ```
        sudo systemctl stop docker
        sudo systemctl stop docker.socket
        sudo apt-get purge docker-ce docker-ce-cli containerd.io docker-compose-plugin
        curl -fsSL https://get.docker.com -o /tmp/get-docker.sh
        sudo bash /tmp/get-docker.sh
        ```

        """)


def check_init():
    check_docker()
    if not is_initialized():
        err("{} does not exist, please run \"vuln-runner init\" first".format(vuln_home()))


def vuln_update():
    for V in VULN_REPOS:
        log("Update {} repo in {}".format(V.repo, V.home()))
        g = git.cmd.Git(V.home())
        g.pull()


def is_initialized():
    return os.path.exists(vuln_home()) and len(os.listdir(vuln_home())) != 0

    # def name(self, vulnerable_name):
    #     return "{}.{}".format(self.name, vulnerable_name)


class Vuln:
    def __init__(self, name, path):
        self.name = name
        self.path = path
        self.compose = None

    def equal(self, name):
        """Returns true when the compose name is equal to the vuln name

        Args:
            name (str): The vuln name

        Returns:
            bool: True when the vuln is equal
        """
        return self.name == name

    def load_compose(self):
        """Load the docker-compose object
        """
        real_path = self.path.replace("VULN_HOME", vuln_home())
        if self.compose is None:
            with open(real_path) as stream:
                self.compose = safe_load(stream)

    def down(self):
        docker = DockerClient(compose_files=[self.path])
        docker.compose.down()

    def run(self):
        docker = DockerClient(compose_files=[self.path])
        docker.compose.up(detach=True)

    def ports(self):
        """Returns the list of ports

        Returns:
            list<port>: The list of ports
        """
        self.load_compose()
        list_ports = []
        for s, value in self.compose['services'].items():
            if 'ports' in value:
                ports = value['ports']
                # Replace the ports with the new port
                for p in ports:
                    splitted = p.split(":")
                    binded = int(splitted[0])
                    second = splitted[1]
                    # If the new port was found
                    list_ports.append(binded)
        return list_ports

    def to_obj(self):
        """Returns a json representation of a vuln

        Returns:
            json: A json representation of the vuln object
        """
        return {
            "name": self.name,
            "path": self.path,
            "ports": self.ports()
        }


class Vulnenv:
    def __init__(self, vulns):
        self.envs = {}
        self.vulns = vulns

    def add_env(self,  env_id, vulns):
        """Add a new enviornment

        Args:
            vulh_obj (Vuln): A Vuln object
        """
        self.envs[env_id] = [v.to_obj() for v in vulns]

    def create_env(self, env_id, no_vulns, virtual=False):
        """Collect the list of vulnerabilities,
        create an env and add to the internal structure

        Args:
            env_id (_type_): _description_
            no_vulns (_type_): _description_
            all_vulns (_type_): _description_
        """
        collected_vulns = []
        busy_ports = []
        no_collected = 0
        for v in self.vulns:
            new_ports = v.ports()
            test_ports = busy_ports + new_ports
            if not get_duplicates(test_ports):
                busy_ports.extend(v.ports())
                no_collected = no_collected + 1
                collected_vulns.append(v)
                self.vulns.remove(v)
            if no_collected == no_vulns:
                break
        if len(collected_vulns) == 0:
            raise Exception(
                "Not available envs. Try with lesser environments or ports")
        self.add_env(env_id, collected_vulns)

    def to_json(self):
        return json.dumps({
            "envs":  self.envs
        }, indent=2)



import os
import json
from docker_vuln_runner.vuln import vuln_home
from docker_vuln_runner.helper import *

"""Manage the hosts file
"""

HOSTS_FILE = "hosts.json"


def hosts_file():
    return os.path.join(vuln_home(), HOSTS_FILE)


def hosts_exists():
    return os.path.exists(hosts_file())


def hosts_init(token):
    h = Hosts(token)
    with open(hosts_file(), 'w') as f:
        f.write(h.to_json())


def hosts_read():
    with open(hosts_file(), 'r') as f:
        file_content = f.read()
        try:
            json_content = json.loads(file_content)
            token = json_content['token']
            envs = json_content['envs']
            h = Hosts(token, envs)
            return h

        except FileNotFoundError:
            json_content = None
        except json.decoder.JSONDecodeError:
            # Cannot read the file
            json_content = None


def token_initialized():
    try:
        h = hosts_read()
        return h
    except Exception:
        return False


class Hosts:
    def __init__(self, token, envs={}):
        self.token = token
        self.envs = envs

    def add_host(self, host):
        self.envs[host] = []

    def to_json(self):
        return json.dumps(self.__dict__, indent=2)

    def update(self):
        with open(hosts_file(), 'w') as f:
            f.write(self.to_json())



import typer
from docker_vuln_runner.vuln import Vulnenv, is_initialized, vuln_home, find_vuln_projects
from docker_vuln_runner.helper import err, log, success, check_network
from docker_vuln_runner.hosts import *
from docker_vuln_controller.client import *


def init_callback(v):
    print("Version: {}".format(v))


typer_app = typer.Typer(add_completion=False, context_settings={
                        "help_option_names": ["-h", "--help"]})


def app():
    typer_app()


@typer_app.command()
def init():
    # token: str=  typer.Option(..., prompt="Setup the token", confirmation_prompt=False, hide_input=True, callback=init_callback)):
    init_confirm = True
    if hosts_exists():
        init_confirm = typer.confirm(
            "hosts already present, are you sure to init?")
    if init_confirm:
        token = typer.prompt("Setup the token", hide_input=True)
        hosts_init(token)
        log("hosts.json created!")

    pass


@typer_app.callback()
def check_initialization():
    if not is_initialized():
        err("{} does not exist, please run \"vuln-runner init\" first".format(vuln_home()))


def check_token():
    h = token_initialized()
    if not h:
        err("Token not initialized, please run \"vuln-controller init\" first")
    return h


@typer_app.command()
def discovery(subnet: str, update_hosts: bool = typer.Option(False, "--update-hosts", "-u")):
    hosts = check_token()
    ipa = check_network(subnet)
    found_hosts = []

    if not ipa:
        err("Please setup a valid network: {}".format(subnet))
    for h in ipa.hosts():
        vuln_client = VulnClient(str(h), hosts.token)
        # log("ping {}".format(h))
        if vuln_client.ping():
            success("{} is a vuln-node!".format(str(h)))
            found_hosts.append(str(h))
    log("Found hosts : {}".format(found_hosts))
    if update_hosts:
        log("Update hosts")
        for hh in found_hosts:
            hosts.add_host(hh)
        hosts.update()
        success("hosts.json updated!")


@typer_app.command()
def generate_vulnenv(no_vulns: int):
    hosts = check_token()
    vulhubs = find_vuln_projects(vuln_home(), True)
    vuln_env = Vulnenv(vulhubs)
    try:
        for h in hosts.envs.keys():
            vuln_env.create_env(h, no_vulns)

    except Exception as e:
        err(e)
    hosts.envs = vuln_env.envs
    hosts.update()
    success("vulnerable environments created in hosts.json ")


@typer_app.command()
def run_env(host: str):
    h = check_token()
    vuln_client = VulnClient(host, h.token)
    envs = h.envs
    env_host = envs.get(host)
    if not env_host:
        err("vulnenv not found for {}".format(host))
    vuln_client.run(env_host)


@typer_app.command()
def run_envs():
    h = check_token()
    envs = h.envs
    hosts = envs.keys()
    for host in hosts:
        env_host = envs.get(host)
        vuln_client = VulnClient(host, h.token)
        vuln_client.run(env_host)


@typer_app.command()
def down_envs():
    h = check_token()
    envs = h.envs
    hosts = envs.keys()
    for host in hosts:
        env_host = envs.get(host)
        vuln_client = VulnClient(host, h.token)
        vuln_client.down(env_host)


@typer_app.command()
def ping(ip: str):
    h = check_token()
    vuln_client = VulnClient(ip, h.token)
    is_node = vuln_client.ping()
    if is_node:
        success("{} is a vuln-node!".format(ip))
    else:
        log("vuln-node not found")



import sys
import pkg_resources
import ipaddress
from os import path
from datetime import datetime

VERSION = pkg_resources.get_distribution('vuln-runner').version

# Colors
SUCCESS_C = '\033[92m'
DEBUG_C = '\033[93m'
ERROR_C = '\033[91m'
BOLD = '\033[1m'
END_C = '\033[0m'


def banner(silent=False):
    if not silent:
        print(
            '''
        ███████╗███████╗ ██████╗███████╗██╗
        ██╔════╝██╔════╝██╔════╝██╔════╝██║
        ███████╗█████╗  ██║     ███████╗██║
        ╚════██║██╔══╝  ██║     ╚════██║██║
        ███████║███████╗╚██████╗███████║██║
        ╚══════╝╚══════╝ ╚═════╝╚══════╝╚═╝
        vuln-runner v{} - https://github.com/cybersecsi/docker-vuln-runner
        '''.format(VERSION))


def log(msg, silent=False):
    if not silent:
        now = datetime.now()
        current_time = now.strftime("%H:%M:%S")
        print(f"[{current_time}] - [LOG] - {msg}", flush=True)


def warn(msg):
    now = datetime.now()
    current_time = now.strftime("%H:%M:%S")
    print(f"{DEBUG_C}[{current_time}] - [WARN] - {msg}{END_C}", flush=True)


def success(msg):
    now = datetime.now()
    current_time = now.strftime("%H:%M:%S")
    print(
        f"{SUCCESS_C}[{current_time}] - [SUCCESS] - {msg}{END_C}", flush=True)


def err(msg):
    now = datetime.now()
    current_time = now.strftime("%H:%M:%S")
    print(f"{ERROR_C}[{current_time}] - [ERROR] - {msg}{END_C}", flush=True)
    sys.exit(-1)


def bold(msg):
    print(f"{BOLD}{msg}{END_C}", flush=True)


def get_compose_name(full_path):
    base_name = path.basename(path.normpath(path.dirname(full_path)))
    return base_name


def check_network(subnet):
    try:
        ipa = ipaddress.ip_network(subnet)
        return ipa

    except Exception as e:
        return False



import socket
from threading import Thread
from docker_vuln_runner.helper import log, warn, VERSION
from docker_vuln_runner.vuln import vuln_home, run_vuln, find_vuln_projects, down_vuln
from docker_vuln_runner.message import VulnMessage


NODE_PORT = 4545


class VulnNode:
    def __init__(self, hosts_obj):
        self.sock = socket.socket()
        self.sock.bind(('0.0.0.0', NODE_PORT))
        self.sock.listen(5)
        self.vulhubs = find_vuln_projects(vuln_home())
        self.hosts_obj = hosts_obj
        log("vuln-node v{} started".format(VERSION))
        while True:
            c, addr = self.sock.accept()
            log('got connection from {}'.format(addr))

            jsonReceived = c.recv(1024)

            # log("Json received --> {}".format(jsonReceived))
            vuln_message = VulnMessage.parse_json(jsonReceived)
            self.dispatch_msg(c, vuln_message)
            c.close()

    def dispatch_msg(self, c, vuln_message):
        received_token = vuln_message.token
        if received_token != self.hosts_obj.token:
            warn("Invalid token received")
            c.send("UNAUTHORIZED".encode('utf-8'))
            return

        # if received_token !=
        if vuln_message.is_hello():
            c.send("vuln-runner v{}".format(VERSION).encode('utf-8'))

        elif vuln_message.is_run():
            vuln_envs = vuln_message.data
            threads = [Thread(target=run_vuln, args=(
                self.vulhubs, v['name'])) for v in vuln_envs]

            # TBD: Insert TODO
            for thread in threads:
                thread.start()

            c.send("STARTED".encode('utf-8'))

        elif vuln_message.is_down():
            vuln_envs = vuln_message.data
            threads = [Thread(target=down_vuln, args=(
                self.vulhubs, v['name'])) for v in vuln_envs]

            # TBD: Insert TODO
            for thread in threads:
                thread.start()

            c.send("STOPPED".encode('utf-8'))

            # for thread in threads:
            #     thread.join()

    def shutdown(self):
        self.sock.shutdown(socket.SHUT_RDWR)
        self.sock.close()



import typer
from docker_vuln_controller.main import check_initialization
from docker_vuln_runner.vuln import is_initialized, vuln_home
from docker_vuln_runner.helper import err, log
from docker_vuln_runner.hosts import *
from docker_vuln_node.node import *

typer_app = typer.Typer(add_completion=False, context_settings={
                        "help_option_names": ["-h", "--help"]})


def app():
    typer_app()


@typer_app.command()
def init():
    # token: str=  typer.Option(..., prompt="Setup the token", confirmation_prompt=False, hide_input=True, callback=init_callback)):
    init_confirm = True
    if hosts_exists():
        init_confirm = typer.confirm(
            "hosts already present, are you sure to init?")
    if init_confirm:
        token = typer.prompt("Setup the token", hide_input=True)
        hosts_init(token)
        log("hosts.json created!")


@typer_app.callback()
def check_initialization():
    if not is_initialized():
        err("{} does not exist, please run \"vuln-runner init\" first".format(vuln_home()))


def check_token():
    h = token_initialized()
    if not h:
        err("Token not initialized, please run \"vuln-node init\" first")
    return h


# Start the vuln-node
@typer_app.command()
def start():
    check_initialization()
    h = check_token()
    VulnNode(h)



import socket
import json
from docker_vuln_runner.helper import err, log, warn, VERSION

NODE_PORT = 4545
TIMEOUT_SOCKET = 0.0001
RUN_TIMEOUT_SOCKET = 1


def check_unauth(ret):
    print(ret)
    if ret == "UNAUTHORIZED":
        err("Invalid token")


def _msg(token, type, data=None):
    if data:
        return json.dumps({
            "token":  token,
            "msg": type,
            "data": data
        })

    else:
        return json.dumps({
            "token":  token,
            "msg": type,
        })


def hello(token):
    return _msg(token, "hello")


def run(token, cves):
    return _msg(token, "run", cves)


def down(token, cves):
    return _msg(token, "down", cves)


def parse_version(version_string):
    v_number = version_string.split()[1].replace('v', '')
    return v_number


class VulnClient:
    def __init__(self, ip, token):
        try:
            self.sock = socket.socket()
            self.sock.settimeout(TIMEOUT_SOCKET)
            self.token = token

        except socket.error as err:
            err('Socket error because of %s' % (err))
        self.address = ip

    def ping(self):
        ret = False
        try:
            self.sock.connect((self.address, NODE_PORT))
            self.sock.send(hello(self.token).encode('utf-8'))
            # Receive
            version_string = self.sock.recv(1024).decode()
            check_unauth(version_string)
            version_number = parse_version(version_string)
            if version_number == VERSION:
                ret = True
            else:
                warn("vuln-node found but different version. Controller version: {}, node version:{}".format(VERSION, version_number))

        except socket.gaierror:
            err('There an error resolving the host')

        except socket.timeout:
            pass
        except Exception as e:
            warn("Error in version detection")

        self.sock.close()
        return ret

    def run(self, cves):
        ret = False
        try:
            self.sock.settimeout(RUN_TIMEOUT_SOCKET)
            self.sock.connect((self.address, NODE_PORT))
            self.sock.send(run(self.token, cves).encode('utf-8'))
            # Receive
            ret = self.sock.recv(1024).decode()
            check_unauth(ret)
            if ret == "STARTED":
                log("{} vulnenv started".format(self.address))

        except socket.gaierror:
            err('There an error resolving the host')

        except Exception as e:
            err("Communication error: {}".format(e))

        self.sock.close()
        return ret

    def down(self, cves):
        ret = False
        try:
            self.sock.settimeout(RUN_TIMEOUT_SOCKET)
            self.sock.connect((self.address, NODE_PORT))
            self.sock.send(down(self.token, cves).encode('utf-8'))
            # Receive
            ret = self.sock.recv(1024).decode()
            check_unauth(ret)
            if ret == "STOPPED":
                log("{} vulnenv stopped".format(self.address))

        except socket.gaierror:
            err('There an error resolving the host')

        except Exception as e:
            err("Communication error: {}".format(e))

        self.sock.close()
        return ret



import json


class VulnMessage:
    def __init__(self, token, msg, data=None):
        self.token = token
        self.msg = msg
        self.data = data

    def parse_json(json_data):
        obj = json.loads(json_data)
        if 'data' in obj.keys():
            return VulnMessage(obj['token'], obj['msg'], obj['data'])
        else:
            return VulnMessage(obj['token'], obj['msg'])

    def is_hello(self):
        return self.msg == 'hello'

    def is_run(self):
        return self.msg == 'run'

    def is_down(self):
        return self.msg == 'down'

    def msg(self):
        return json.dumps(self.__dict__)
